/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.message.notifier;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import javax.swing.ImageIcon;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.message.api.Message;
import nl.cloudfarming.client.message.api.MessageApiModule;
import nl.cloudfarming.client.message.api.MessageEvent;
import nl.cloudfarming.client.message.api.MessageEventKey;
import nl.cloudfarming.client.message.api.MessageEventListener;
import nl.cloudfarming.client.message.singlemessage.SingleMessageFrame;

import nl.cloudfarming.eventbus.GuiEvent;
import nl.cloudfarming.eventbus.GuiEventKey;
import nl.cloudfarming.eventbus.GuiEventListener;
import org.openide.awt.Notification;
import org.openide.awt.NotificationDisplayer;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;
import org.openide.util.lookup.ServiceProviders;

/**
 * Wrapperclass to show the NotificationDisplayer when GuiEvent for a new incoming Message is triggered.
 * Keep track of messages and their notifications to purge the notification from the list if the message has been read.
 * 
 * @author Gerben Feenstra
 */
@ServiceProviders(value = {
    @ServiceProvider(service = GuiEventListener.class)
})
public class MessageNotifier extends MessageEventListener {

    // Keep track of messages and their notifications to purge the notification from the list if the corresponding message has been read.
    private static HashMap<Message, Notification> activeNotifications = new HashMap<Message, Notification>();
    private static final AppLogger LOG = AppLogFactory.getLogger(MessageNotifier.class);
    
    // Get global message dateformat from API
    private static final String DATE_FORMAT = NbBundle.getMessage(MessageApiModule.class, "message.api.datereceived.format");

    // Service providers must have a public no-arg constructor
    public MessageNotifier() {
    }

    /**
     * A message has been read and the notification belonging to that message can be cleared and removed.
     * If the event contains a message for which there is no notifier nothing will happen.
     * 
     * @param event Event fired by the GuiEventBus containing the message
     */
    void removeNotifier(GuiEvent<Message> event) {
        final Message message = event.getContent();

        //Log: Notification for message has been removed
        LOG.info(NbBundle.getMessage(MessageNotifierModule.class, "message.notifier.log.notification_removed", message.getId()));

        Notification notificationToDisable = activeNotifications.get(message);
        if (notificationToDisable != null) {
            notificationToDisable.clear();
            activeNotifications.remove(message);
        } else {
            // Log: Remove of notification failed
            LOG.error(NbBundle.getMessage(MessageNotifierModule.class, "message.notifier.log.notification_remove_failed", message.getId()));
        }
    }

    /**
     * Retrieve Message from event and show the NotificationDisplayer with the message-info provided.
     * Clicking the notifier-balloon will open the inbox topcomponent
     * @param event Event fired by the GuiEventBus.
     */
    void showNotifier(GuiEvent<Message> event) {
        final Message message = event.getContent();

        //Log: Notification for message has been removed
        LOG.info(NbBundle.getMessage(MessageNotifierModule.class, "message.notifier.log.message_received", message.getId()));

        StringBuilder dateSb = new StringBuilder();
        dateSb.append(new SimpleDateFormat(DATE_FORMAT).format(message.getDateReceived()));
        dateSb.append(" - ");
        // Who/what send the message?
        dateSb.append(NbBundle.getMessage(MessageNotifierModule.class, "message.notifier.message.origin.label.text", message.getOrigin()));

        // Subject as clickable part of the notifier
        String clickableText = NbBundle.getMessage(MessageNotifierModule.class, "message.notifier.notification.text.clickable", message.getTitle());

        Notification notification = (Notification) NotificationDisplayer.getDefault().notify(dateSb.toString(), new ImageIcon(), clickableText, new ActionListener() {

            // Define action when balloon is clicked
            @Override
            public void actionPerformed(ActionEvent e) {
                SingleMessageFrame messagePopUp = new SingleMessageFrame(message);
                messagePopUp.setVisible(true);
            }
        });
        activeNotifications.put(message, notification);
    }

    /**
     * Indicate if a notification belonging to the specified message is still active
     * @param message 
     */
    public static boolean isActive(Message message) {
        if (activeNotifications.containsKey(message)) {
            return true;
        }
        return false;
    }
    
    @Override
    public void onEvent(MessageEvent event) {
        if (MessageEventKey.MESSAGE_NEW.equals(event.getKey())) {
            //Notify everybody 
            showNotifier(event);
        } else if (MessageEventKey.MESSAGE_READ.equals(event.getKey())) {
            removeNotifier(event);
        } 
    }

    @Override
    public boolean listensTo(GuiEventKey guiEventKey) {
        return MessageEventKey.MESSAGE_NEW.equals(guiEventKey) || MessageEventKey.MESSAGE_READ.equals(guiEventKey);
    }

    @Override
    public String getModuleName() {
        return MessageNotifierModule.MODULE_NAME;
    }

    @Override
    public boolean listensToMessageType(Message message) {
        return true;
    }

    /**
     * Notifier module does not persist
     * 
     * @param message
     * @return 
     */
    @Override
    public boolean persist(Message message) {
        return false;
    }

}
